package org.adangel.simplegraphicsgenerator;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.paint.Color;

public class SimpleGraphicsGeneratorController {
    @FXML private Canvas canvas;

    @FXML protected void handleGenerateButton(ActionEvent event) {
        GraphicsContext gc = canvas.getGraphicsContext2D();
        draw(gc);
    }

    private final Random random = new Random();

    private void draw(GraphicsContext gc) {
        gc.setFill(Color.WHITE);
        gc.setStroke(Color.WHITE);
        gc.fillRect(0, 0, canvas.getWidth(), canvas.getHeight());

        int width = (int)canvas.getWidth();
        int height = (int)canvas.getHeight();

        Map<String, Object> visited = new HashMap<>();

        // start location
        int x = random.nextInt(width);
        int y = random.nextInt(height);
        int distance = 10;

        gc.setStroke(Color.BLACK);
        gc.beginPath();
        gc.moveTo(x, y);

        visited.put(x + "-" + y, Boolean.TRUE);

        Direction lastDirection = null;
        for (int i = 0; i < 1000; i++) {
            List<Direction> availableDirections = new ArrayList<>(Arrays.asList(Direction.values()));
            if (lastDirection != null) availableDirections.remove(lastDirection.back());


            Direction nextDirection = availableDirections.get(random.nextInt(availableDirections.size()));
            int nextX = nextDirection.x(x, distance);
            int nextY = nextDirection.y(y, distance);

            do {
                if (nextX < 0) nextX = 0;
                if (nextX > width) nextX = width;
                if (nextY < 0) nextY = 0;
                if (nextY > height) nextY = height;

                if (!visited.containsKey(nextX + "-" + nextY)) {
                    break;
                }

                availableDirections.remove(nextDirection);
                if (availableDirections.size() > 1) {
                    nextDirection = availableDirections.get(random.nextInt(availableDirections.size()));
                } else if (availableDirections.size() == 1) {
                    nextDirection = availableDirections.get(0);
                } else {
                    nextDirection = lastDirection;
                }
                nextX = nextDirection.x(x, distance);
                nextY = nextDirection.y(y, distance);
            } while (!availableDirections.isEmpty());

            gc.lineTo(nextX, nextY);

            lastDirection = nextDirection;
            x = nextX;
            y = nextY;
            visited.put(x + "-" + y, Boolean.TRUE);
        }
        gc.stroke();
    }

    private enum Direction {
        UP {
            @Override
            public int y(int y, int distance) {
                return y - distance;
            }
            @Override
            public Direction back() { return DOWN; }
        },
        DOWN {
            @Override
            public int y(int y, int distance) {
                return y + distance;
            }
            @Override
            public Direction back() { return UP; }
        },
        LEFT {
            @Override
            public int x(int x, int distance) {
                return x - distance;
            }
            @Override
            public Direction back() { return RIGHT; }
        },
        RIGHT {
            @Override
            public int x(int x, int distance) {
                return x + distance;
            }
            @Override
            public Direction back() { return LEFT; }
        };

        public int x(int x, int distance) {
            return x;
        }
        public int y(int y, int distance) {
            return y;
        }
        public abstract Direction back();
    }
}
